(******************************************************************************
 * ACCHeck                                                                    *
 ******************************************************************************)
unit ACCheck;

(******************************************************************************
 * Interface                                                                  *
 ******************************************************************************)
Interface

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
 uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus;

(******************************************************************************
 * Type                                                                       *
 ******************************************************************************)
 type
  TACCheck = class(TCustomControl)
  private
    FDown:Boolean;
    FState:TCheckBoxState;
    FFocused:Boolean;
    FCheckColor:TColor;
  protected
    constructor Create(AOwner: TComponent); override;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState;X, Y: Integer);
      override;
    procedure KeyDown(var Key:Word;Shift:TShiftSTate); override;
    procedure KeyUp(var Key:Word;Shift:TShiftSTate); override;
    procedure SetDown(Value:Boolean);
    procedure SetState(Value:TCheckBoxState);
    procedure SetChecked(Value:Boolean);
    function  GetChecked:Boolean;
    procedure SetCheckColor(Value:TColor);
    function  GetCaption: TCaption;
    procedure SetCaption(const Value:TCaption);
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure disegna(left,top,bottom:Integer;Tipo:Integer);
  public
  published
    property Caption:TCaption read GetCaption write SetCaption;
    property CheckColor:TColor read FCheckColor write SetCheckColor
             default clWindowText;
    property Checked:Boolean read GetChecked write SetChecked
             default False;
    property Down:Boolean read FDown write SetDown default False;
    property DragCursor;
    property DragMode;
    property Font;
    property ParentFont;
    property ParentColor;
    property PopupMenu;
    property ShowHint;
    property State:TCheckBoxState read FState write SetState
             default cbUnchecked;
    property TabOrder;
    property TabStop;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;


(******************************************************************************
 * ACCHeck                                                                    *
 ******************************************************************************)
 implementation

(******************************************************************************
 * ACCHeck                                                                    *
 ******************************************************************************)
constructor TACCheck.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 98;
  Height := 20;
  ParentColor:=False;
  Color:=clBtnFace;
  ParentColor:=True;
end;

(******************************************************************************
 * ACCHeck                                                                    *
 ******************************************************************************)
const BWidth=12;


(******************************************************************************
 * Disegna                                                                    *
 *                                                                            *
 *                                                                            *
 * Nome          Descrizione                                                  *
 * ------------- ------------------------------------------------------------ *
 * Left          Colonna in cui disegnare                                     *
 * Top           Riga in cui cominciare a tracciare la linea                  *
 * Bottom        Riga in cui terminare di tracciare la linea                  *
 * Tipo          Stile della linea                                            *
 *                   0: Linea continua                                        *
 ******************************************************************************)
procedure TACCheck.disegna(left,top,bottom:Integer;Tipo:Integer);
var c1 : Integer;
    fl : Boolean;
begin
 case tipo of
   0:
     begin
       for c1 := top to bottom do
       begin
         Canvas.Pixels[Left,c1]:=Canvas.Pen.Color;
       end;
     end;
   1:
     begin
       fl := ((left and 1) <> 0); // in questo modo le linee dispari inizieranno
                                  // con un punto pieno e quelle pari con un
                                  // punto vuoto
       for c1 := top to bottom do
       begin
         if (fl) then Canvas.Pixels[Left,c1]:=Canvas.Pen.Color;
         fl := not fl;
       end;
     end;
   end;
end;

(******************************************************************************
 * Paint                                                                      *
 ******************************************************************************)
procedure TACCheck.Paint;
var BLeft,BTop,BRight,BBottom:Integer;
    TX,TY,TW,TH:Integer;
    Rect:TRect;
    Conta1 : Integer;

begin
  Canvas.Font:=Font;
  with Canvas do
  begin
    BTop:=(Height div 2)-(BWidth div 2);
    BBottom:=BTop+BWidth;
    BLeft:=1;
    BRight:=BWidth+1;
    //Brush.Color:=clBtnFace;
    Brush.Color := Canvas.Font.Color;
    Pen.Color := Canvas.Font.Color;
    if not FDown then
    begin
      Brush.Style := bsClear;
      Pen.Width:=1;
      Rectangle(BLeft,BTop,BRight,BBottom);
    end
    else
    begin
      Brush.Style := bsClear;
      Pen.Width:=2;
      Rectangle(BLeft+1,BTop,BRight+1,BBottom+1);
      Pen.Width:=1;
    end;
    TX:=BRight+5;
    TY:=(Height div 2)+(Font.Height div 2)-1;
    TW:=TextWidth(Caption);
    TH:=TextHeight(Caption);
    TextOut(TX,TY,Caption);
    case State of
      cbChecked:
        begin
          Pen.Color:=CheckColor;
          Pen.Width:=1;
          Disegna(BLeft+1,  BTop+3,  BTop+4,  0);
          Disegna(BLeft+2,  BTop+3,  BTop+6,  0);
          Disegna(BLeft+3,  BTop+4,  BTop+8,  0);
          Disegna(BLeft+4,  BTop+5,  BTop+10, 0);
          Disegna(BLeft+5,  BTop+5,  BTop+10, 0);
          Disegna(BLeft+6,  BTop+4,  BTop+8,  0);
          Disegna(BLeft+7,  BTop+3,  BTop+6,  0);
          Disegna(BLeft+8,  BTop+2,  BTop+5,  0);
          Disegna(BLeft+9,  BTop+0,  BTop+3,  0);
          Disegna(BLeft+10, BTop+0,  BTop+2,  0);
          Disegna(BLeft+11, BTop-1,  BTop+1,  0);
        end;
       cbGrayed:
        begin
          Pen.Color:=CheckColor;
          if Down then
          begin
            Brush.Color:=clBtnFace;
            Brush.Style:=bsClear;
            Rectangle(BLeft+2,BTop+2,BRight-1,BBottom-1);
          end;
          For Conta1 := 1 to 10 do
             Disegna (BLeft+Conta1, Btop+1, BBottom-2, 1);
        end;
    end;
    Brush.Color:=clBtnFace;
    if FFocused then
    begin
      Rect:=Bounds(TX-1,TY,TW+2,TH+0);
      FrameRect(Rect);
      DrawFocusRect(Rect);
    end;
  end;
end;

(******************************************************************************
 * GetCaption                                                                 *
 ******************************************************************************)
function TACCheck.GetCaption:TCaption;
var Buf:array[0..256] of Char;
begin
   GetTextBuf(Buf,256);
   Result:=StrPas(Buf);
end;

(******************************************************************************
 * ACCHeck                                                                    *
 ******************************************************************************)
procedure TACCheck.SetCaption(const Value:TCaption);
var Buffer: array[0..255] of Char;
begin
  if GetCaption <> Value then
    SetTextBuf(StrPCopy(Buffer,Value));
  Invalidate;
end;

(******************************************************************************
 * SetDown                                                                    *
 ******************************************************************************)
procedure TACCheck.SetDown(Value:Boolean);
begin
 if FDown<>Value then
   begin
     FDown:=Value;
     Invalidate;
   end;
end;

(******************************************************************************
 * SetState                                                                   *
 ******************************************************************************)
procedure TACCheck.SetState(Value:TCheckBoxState);
begin
 if FState<>Value then
 begin
   FState:=Value;
   Invalidate;
   Click;
 end;
end;

(******************************************************************************
 * GetChecked                                                                 *
 ******************************************************************************)
function TACCheck.GetChecked: Boolean;
begin
  Result:=State=cbChecked;
end;

(******************************************************************************
 * SetChecked                                                                 *
 ******************************************************************************)
procedure TACCheck.SetChecked(Value:Boolean);
begin
  if Value then State := cbChecked
           else State := cbUnchecked;
  Invalidate;
end;

(******************************************************************************
 * SetCheckColor                                                              *
 ******************************************************************************)
procedure TACCheck.SetCheckColor(Value:TColor);

begin
  FCheckColor:=Value;
  Invalidate;
end;

(******************************************************************************
 * DoEnter                                                                    *
 ******************************************************************************)
procedure TACCheck.DoEnter;
begin
  inherited DoEnter;
  FFocused:=True;
  Invalidate;
end;

(******************************************************************************
 * DoExit                                                                     *
 ******************************************************************************)
procedure TACCheck.DoExit;

begin
     inherited DoExit;
     FFocused:=False;
     Invalidate;
end;

(******************************************************************************
 * MouseDown                                                                  *
 ******************************************************************************)
procedure TACCheck.MouseDown(Button: TMouseButton; Shift: TShiftState;
                                  X, Y: Integer);

begin
     SetFocus;
     FFocused:=True;
     inherited MouseDown(Button, Shift, X, Y);
     MouseCapture:=True;
     Down:=True;
end;

(******************************************************************************
 * MouseUp                                                                    *
 ******************************************************************************)
procedure TACCheck.MouseUp(Button: TMouseButton; Shift: TShiftState;
                                  X, Y: Integer);

begin
     MouseCapture:=False;
     Down:=False;
     if (X>=0) and (X<=Width) and (Y>=0) and (Y<=Height) then
       Checked:=not Checked;
     inherited MouseUp(Button, Shift, X, Y);
     Invalidate;
end;

(******************************************************************************
 * MouseMove                                                                  *
 ******************************************************************************)
procedure TACCheck.MouseMove(Shift: TShiftState;X, Y: Integer);

begin
  inherited MouseMove(Shift,X,Y);
end;

(******************************************************************************
 * KeyDown                                                                    *
 ******************************************************************************)
procedure TACCheck.KeyDown(var Key:Word;Shift:TShiftSTate);
begin
  if Key=vk_Space then Down:=True;
  inherited KeyDown(Key,Shift);
end;

(******************************************************************************
 * KeyUp                                                                      *
 ******************************************************************************)
procedure TACCheck.KeyUp(var Key:Word;Shift:TShiftSTate);
begin
  if Key=vk_Space then
   begin
     Down:=False;
     Checked:=not Checked;
   end;
end;

end.
