(******************************************************************************
 * TACLinkCreate                                                              *
 *                                                                            *
 * Because CreateLink calls the CoCreateInstance function, it is assumed      *
 * that the CoInitialize function has already been called. CreateLink uses    *
 * the IPersistFile interface to save the shortcut and the IShellLink         *
 * interface to store the filename and description.                           *
 *                                                                            *
 * Before:                                                                    *
 *	   CoInitialize(nil)                                                  *
 *                                                                            *
 * After:                                                                     *
 *         CoUnInitialize                                                     *
 *                                                                            *
 ******************************************************************************)
unit ACLinkCreate;
(******************************************************************************
 * Interface                                                                  *
 ******************************************************************************)
interface

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

(******************************************************************************
 * TAcCreateLink                                                              *
 ******************************************************************************)
type
  TACCreateLink = class(TComponent)
  private
    { Private declarations }
      ftarget:string;
      fDescription:string;
      fWorkingDir:string;
      fArguments:string;
      fHotKey:Integer;
      fShowCmd:Integer;
      fIconLocation:String;
      fIconNumber:Integer;
      fLinkName:String;

  protected
    { Protected declarations }
  public
    { Public declarations }
    Function Execute:Boolean;
    Function Read:Boolean;
  published
    { Published declarations }
    Property Target : String Read ftarget Write ftarget;
    Property Description : String Read fDescription Write fDescription;
    Property WorkingDir : String Read fWorkingDir Write fWorkingDir;
    Property Arguments : String Read fArguments Write fArguments;
    Property HotKey : Integer Read fHotKey Write fHotKey;
    Property ShowCmd : Integer Read fShowCmd Write fShowCmd;
    Property IconLocation : String Read fIconLocation Write fIconLocation;
    Property IconNumber : Integer Read fIconNumber Write fIconNumber;
    Property LinkName : String Read fLinkName Write fLinkName;
  end;

(******************************************************************************
 * Implementation                                                             *
 ******************************************************************************)
implementation

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
Uses ole2, ShlObj;

(******************************************************************************
 * Execute                                                                    *
 *                                                                            *
 * Ritorna                                                                    *
 *   True: Ok                                                                 *
 *   False: Errore                                                            *
 ******************************************************************************)
Function TACCreateLink.Execute:Boolean;
var Txt1,Txt2 : array[0..255] of char;
    Ris       : HResult;
    Psl       : IShellLink;
    Ppf       : IPersistFile;
    Saver     : Array [0..Max_Path] of WideChar;

begin
  if (CoCreateInstance(CLSID_ShellLink,
                       nil,
                       CLSCTX_INPROC_SERVER,
                       IID_IShellLink,psl)<>0) then
  begin
    Execute := False;
    Exit;
  end;

  try
    StrPCopy(txt1,Target);
    if (psl.SetPath(PChar(fTarget))<>0) then
    begin
      Execute := False;
      Exit;
    end;

    StrPCopy(txt1,Description);
    Ris := 0;
    Ris := Ris + psl.SetDescription(txt1);
    Ris := Ris + psl.SetWorkingDirectory(PChar(WorkingDir));
    Ris := Ris + psl.SetArguments(PChar(Arguments));
    Ris := Ris + psl.SetHotKey(HotKey);
    Ris := Ris + psl.SetShowCmd(ShowCmd);
    Ris := Ris + psl.SetIconLocation(PChar(IconLocation),IconNumber);
    If Ris<>0 then
    begin
      Execute:=False;
      Exit;
    end;


    if (psl.QueryInterface(IID_IPersistFile,ppf)<>0) then
    begin
      Execute := False;
      Exit;
    end;

    StrPCopy(txt2,LinkName);

    MultiByteToWideChar(CP_ACP,0,txt2,-1,Saver,Max_Path);

    if (ppf.Save(Saver,True)<>0) then
    begin
      Execute := False;
      Exit;
    end;
    ppf.release;
  finally
    psl.release;
  end;
  Execute := True;
End;


(******************************************************************************
 * Read                                                                       *
 ******************************************************************************)
Function TACCreateLink.Read:Boolean;
var Txt1      : Array[0..255] of char;
    Ris       : HResult;
    Psl       : IShellLink;
    Ppf       : IPersistFile;
    Saver     : Array [0..Max_Path] of WideChar;
    X1        : Array [0..255] Of Char;
    Data      : TWin32FindData;
    App1      : Integer;
    W         : Word;
begin
  if (CoCreateInstance(CLSID_ShellLink,
                      nil,
                      CLSCTX_INPROC_SERVER,
                      IID_IShellLink,psl)<>0) Then
  begin
    Read:=False;
    Exit;
  end;
  if (psl.QueryInterface(IID_IPersistFile,ppf)<>0) Then
  begin
    Read:=False;
    Exit;
  end;
  StrPCopy(Txt1,LinkName);
  MultiByteToWideChar(CP_ACP,0,Txt1,-1,Saver,Max_Path);
  if (ppf.Load(Saver,STGM_READ)<>0) then
  begin
    Read:=False;
    Exit;
  end;

  if (psl.Resolve(0,SLR_ANY_MATCH)<>0) then
  begin
    Read:=False;
    Exit;
  end;

  if (psl.GetWorkingDirectory(@X1,MAX_PATH )<>0) then
  begin
    Read:=False;
    Exit;
  end;
  
  fWorkingDir  := StrPas(@X1);
  Ris          := psl.GetPath( @X1,MAX_PATH,Data,SLGP_UNCPRIORITY);
  Target       := StrPas(@X1);
  Ris          := Ris + psl.GetIconLocation(@X1,MAX_PATH,App1);
  IconLocation := StrPas(@X1);
  IconNumber   := App1;
  Ris          := Ris + psl.GetDescription(@X1,MAX_PATH );
  Description  := StrPas(@X1);
  App1         := 0;
  Ris          := Ris + psl.GetShowCmd(App1);
  fShowCmd     := App1;
  W            := 0;
  Ris          := Ris + psl.GetHotKey(W);
  fHotKey      := W;
  Ris          := Ris + psl.GetArguments(@X1,MAX_PATH );
  fArguments   := StrPas(@X1);

  ppf.release;
  psl.release;
  Read := (Ris = 0);
end;
end.
