(******************************************************************************
 * AcAssist                                                                   *
 ******************************************************************************)
unit AcAssist;

(******************************************************************************
 * Interface                                                                  *
 ******************************************************************************)
 interface
(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ACDialogEngine;

(******************************************************************************
 * Type                                                                       *
 ******************************************************************************)
type
  TACDialogStyle = (TACDS_Si_No_Annulla,
                    TACDS_Ok,
                    TACDS_Error,
                    TACDS_Si_No);
  TACAssist = class(TComponent)
  private
    { Private declarations }
    Scheda : TForm;
    Immagine : TPicture;
    Frm      : TForm; // Form del fumetto
    F_DialogTitle : String;
    F_DialogText  : String;
    F_Dialog_Style : TACDialogStyle;

  protected
    { Protected declarations }
    constructor Create(AOwner: TComponent); Override;
    Destructor Destroy;Override;
    function Get_Form_Visible : Boolean;
    procedure Set_Form_Visible(fl:Boolean);
    procedure Set_Immagine(img:TPicture);
    procedure Form_Paint(Sender: TObject);
    procedure On_Exit_form(Sender: TObject);
    procedure Riposiziona_controlli(Ctr:TACDialogEngine;
                                    Old_Style, New_Style : ACDEPosition );
    function Value_position_vert(Pos:ACDEPosition):Integer;
    function Value_position_ori(Pos:ACDEPosition):Integer;
    function Get_Left : Integer;
    procedure Set_Left(X:Integer);
    function Get_Top: Integer;
    procedure Set_Top(X:Integer);

  public
    { Public declarations }
    procedure Show_Dialog(Ctr:TACDialogEngine);
    function Show_Modal_Dialog(Ctr:TACDialogEngine):TModalResult;
    function Show_Dialog_Box:TModalResult;
  published
    { Published declarations }
    property Visible : Boolean Read Get_Form_Visible Write Set_Form_Visible;
    property Picture : TPicture Read Immagine Write Set_Immagine;
    property Left    : Integer Read Get_Left Write Set_Left;
    property Top     : Integer Read Get_Top Write Set_Top;
    Property DialogTitle : String Read F_DialogTitle Write F_DialogTitle;
    Property DialogText  : String Read F_DialogText Write F_DialogText;
    property DialogStyle : TACDialogStyle Read F_Dialog_Style Write F_Dialog_Style;


  end;

(******************************************************************************
 * Implementation                                                             *
 ******************************************************************************)
implementation

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
uses ACFormDialog_Si_No_Annulla, AcFormDialog_Ok, ACFormDialog_Error,
     ACFormDialog_Si_No;

(******************************************************************************
 * Constructor                                                                *
 ******************************************************************************)
 constructor TACassist.Create(AOwner: TComponent);
begin
  Inherited;
  Scheda := TForm.Create(Self);
  With scheda do
  begin
    BorderStyle := bsToolWindow;
    OnPaint     := Form_Paint;
    FormStyle   := fsStayOnTop;
    Height      := 10;
    Width       := 10;

  end;
  Immagine := TPicture.Create;

//  Immagine.Parent := Self;
end;

(******************************************************************************
 * Destructor                                                                 *
 ******************************************************************************)
Destructor TACassist.Destroy;
begin
  Scheda.Free;
  Immagine.Free;
  Inherited;

end;

(******************************************************************************
 * Get_Form_Visible                                                           *
 ******************************************************************************)
function TACassist.Get_Form_Visible : Boolean;
begin
  Get_Form_Visible := Scheda.visible;
end;

(******************************************************************************
 * Set_Form_Visible                                                           *
 ******************************************************************************)
procedure TACassist.Set_Form_Visible(fl:Boolean);
begin
  Scheda.visible := fl;
end;

(******************************************************************************
 * Set_Immagine                                                               *
 ******************************************************************************)
procedure TACassist.Set_Immagine(img:TPicture);
Begin
  Immagine.Assign(Img);
End;


(******************************************************************************
 * Form_Paint                                                                 *
 ******************************************************************************)
procedure TACAssist.Form_Paint(Sender: TObject);
Var dy,dx : Integer;
begin
  with Picture.BitMap.Canvas.ClipRect do
  begin
    Dx := -Scheda.ClientWidth+Scheda.Width;
    Dy := -Scheda.ClientHeight+Scheda.Height;
    Scheda.Width := dx+Right-Left;
    Scheda.Height := dy+Bottom-Top;
  End;
  Scheda.Canvas.CopyRect(Picture.BitMap.Canvas.ClipRect,
                         Picture.BitMap.Canvas,
                         Picture.BitMap.Canvas.ClipRect);
end;

(******************************************************************************
 * On_Exit_form                                                               *
 ******************************************************************************)
procedure TACAssist.On_Exit_form(Sender: TObject);
begin
  frm.hide;
end;

(******************************************************************************
 * Show_Dialog                                                                *
 ******************************************************************************)
procedure TACAssist.Show_Dialog(Ctr:TACDialogEngine);
Var
//    x1,              // Posizione X Form
//    y1,              // Posizione Y Form
    l1,              // Larghezza Form
    h1,              // Altezza Form
    ls,              // Larghezza schermo
    hs               // Altezza schermo
          : Integer;
    Old_Style : ACDEPosition;
begin
  if(Not Scheda.visible) then
  begin
    Scheda.BringToFront;
    Scheda.Show;
  end;
  SetWindowPos(scheda.Handle,
               HWND_TOPMOST,
               scheda.left,
               scheda.top,
               scheda.width,
               scheda.height,
               SWP_NOACTIVATE);  
  Frm := TForm(Ctr.Owner);
//  x1 := Frm.Left;
//  y1 := Frm.Top;
  l1 := Frm.Width;
  h1 := Frm.Height;
  ls := Screen.Width;
  hs := Screen.height;
  Old_Style := Ctr.Style;
  Ctr.Style := ACDETopLeft;
  Frm.Left := Scheda.Left+Scheda.Width-30;
  Frm.Top  := Scheda.Top+Scheda.Height-20;
  Ctr.OnExit := On_Exit_form;
  Frm.OnDeactivate := On_Exit_form;

(********** Se il form non ci sta nella posizione *********)
(********** e cambia la freccia                   *********)
  if (Scheda.Top+Scheda.Height-20+Frm.Height>Hs) then
  begin
    Ctr.Style := ACDEBottomLeft;
    Frm.Top   := Scheda.Top-h1+30;
  end;
(********** se non ci sta in larghezza **********)
(********** lo sposto a sinistra e     **********)
(********** metto la freccia a destra  **********)
  if (Scheda.Left+Scheda.Width-20+Frm.Width>Ls) then
  begin
    if (Ctr.Style = ACDEBottomLeft) then Ctr.Style := ACDEBottomRight
    Else Ctr.Style := ACDETopRight;
    Frm.Left   := Scheda.Left-L1+40;
  end;

  Riposiziona_controlli(Ctr,old_Style,Ctr.Style);
  Frm.Show;

end;
(******************************************************************************
 * Show_Modal_Dialog                                                          *
 ******************************************************************************)
function TACAssist.Show_Modal_Dialog(Ctr:TACDialogEngine):TModalResult;
Var
//    x1,              // Posizione X Form
//    y1,              // Posizione Y Form
    l1,              // Larghezza Form
    h1,              // Altezza Form
    ls,              // Larghezza schermo
    hs               // Altezza schermo
          : Integer;
    Old_Style : ACDEPosition;
    Ris       : TModalResult;
begin
  if(Not Scheda.visible) then
  begin
    Scheda.BringToFront;
    Scheda.Show;
  end;
  SetWindowPos(scheda.Handle,
               HWND_TOPMOST,
               scheda.left,
               scheda.top,
               scheda.width,
               scheda.height,
               SWP_NOACTIVATE);  
  Frm := TForm(Ctr.Owner);
//  x1 := Frm.Left;
//  y1 := Frm.Top;
  l1 := Frm.Width;
  h1 := Frm.Height;
  ls := Screen.Width;
  hs := Screen.height;
  Old_Style := Ctr.Style;
  Ctr.Style := ACDETopLeft;
  Frm.Left := Scheda.Left+Scheda.Width-30;
  Frm.Top  := Scheda.Top+Scheda.Height-20;
  Ctr.OnExit := On_Exit_form;
  Frm.OnDeactivate := Nil;

(********** Se il form non ci sta nella posizione *********)
(********** e cambia la freccia                   *********)
  if (Scheda.Top+Scheda.Height-20+Frm.Height>Hs) then
  begin
    Ctr.Style := ACDEBottomLeft;
    Frm.Top   := Scheda.Top-h1+30;
  end;
(********** se non ci sta in larghezza **********)
(********** lo sposto a sinistra e     **********)
(********** metto la freccia a destra  **********)
  if (Scheda.Left+Scheda.Width-20+Frm.Width>Ls) then
  begin
    if (Ctr.Style = ACDEBottomLeft) then Ctr.Style := ACDEBottomRight
    Else Ctr.Style := ACDETopRight;
    Frm.Left   := Scheda.Left-L1+40;
    if(Frm.Left<0) then
    begin
      if (Ctr.Style = ACDEBottomRight) then Ctr.Style := ACDEBottomCenter
      Else Ctr.Style := ACDETopCenter;
      Frm.Left := Scheda.Left+(Scheda.Width div 2)-(frm.Width div 2);
    end;
  end;

  Riposiziona_controlli(Ctr,old_Style,Ctr.Style);
  Ris := Frm.ShowModal;
  Show_Modal_Dialog := ris;


end;

(******************************************************************************
 * Value_position_ver                                                         *
 *                                                                            *
 * La posizione in alto vale: -1                                              *
 * La posizione in basso vale: 1                                              *
 * Le altre valgono: 0                                                        *
 ******************************************************************************)
function TACAssist.Value_position_vert(Pos:ACDEPosition):Integer;
Var v : Integer;
begin
 v := 0;
 if(pos in [ACDETopLeft,ACDETopRight,ACDETopCenter]) Then v:=-1;
 if(pos in [ACDEBottomLeft,ACDEBottomRight,ACDEBottomCenter]) Then v:=1;
 Value_position_vert := v;
end;

(******************************************************************************
 * Value_position_Ori                                                         *
 *                                                                            *
 * La posizione a sinistra vale: -1                                           *
 * La posizione a destra vale: 1                                              *
 * Le altre valgono: 0                                                        *
 ******************************************************************************)
function TACAssist.Value_position_ori(Pos:ACDEPosition):Integer;
Var v : Integer;
begin
 v := 0;
 if(pos in [ACDELeftTop,ACDELeftBottom]) Then v:=-1;
 if(pos in [ACDERightTop,ACDERightBottom]) Then v:=1;
 Value_position_Ori := v;
end;

(******************************************************************************
 * Riposiziona_Controlli                                                      *
 ******************************************************************************)
procedure TACAssist.Riposiziona_controlli(Ctr:TACDialogEngine;
                                          Old_Style, New_Style : ACDEPosition );

Var pos_1, pos_2 : Integer;
    c1             : Integer;
begin
  pos_1 := Value_position_vert(Old_Style);
  pos_2 := Value_position_vert(New_Style);
(********** Se passa da sopra a sotto **********)
(********** alzo i componenti         **********)
  if(Pos_1<Pos_2) then
  begin
    for c1 := 0 to ctr.ControlCount-1 do
    begin
      ctr.Controls[c1].Top := ctr.Controls[c1].Top-20;
    end;
  end;
(********** Se passa da sotto a sopra **********)
(********** abbasso i componenti      **********)
  if(Pos_1>Pos_2) then
  begin
    for c1 := 0 to ctr.ControlCount-1 do
    begin
      ctr.Controls[c1].Top := ctr.Controls[c1].Top+20;
    end;
  end;

  pos_1 := Value_position_ori(Old_Style);
  pos_2 := Value_position_ori(New_Style);
(********** Se passa da sopra a sotto **********)
(********** alzo i componenti         **********)
  if(Pos_1<Pos_2) then
  begin
    for c1 := 0 to ctr.ControlCount-1 do
    begin
      ctr.Controls[c1].Left := ctr.Controls[c1].Left-20;
    end;
  end;
(********** Se passa da sotto a sopra **********)
(********** abbasso i componenti      **********)
  if(Pos_1>Pos_2) then
  begin
    for c1 := 0 to ctr.ControlCount-1 do
    begin
      ctr.Controls[c1].Left := ctr.Controls[c1].Left+20;
    end;
  end;

//  if(old_pos
end;

(******************************************************************************
 * Get_Left                                                                   *
 ******************************************************************************)
function TACAssist.Get_Left : Integer;
begin
  Get_Left := Scheda.Left;
end;
(******************************************************************************
 * Set_Left                                                                   *
 ******************************************************************************)
procedure TACAssist.Set_Left(X:Integer);
begin
  Scheda.Left:=x;
end;
(******************************************************************************
 * Get_Top                                                                    *
 ******************************************************************************)
function TACAssist.Get_Top: Integer;
begin
  Get_Top := Scheda.Top;
end;
(******************************************************************************
 * Set_Top                                                                    *
 ******************************************************************************)
procedure TACAssist.Set_Top(X:Integer);
begin
  Scheda.Top := x;
end;

(******************************************************************************
 * Show_Dialog_Box                                                            *
 ******************************************************************************)
function TACAssist.Show_Dialog_Box:TModalResult;
Var

  Frm1 : TFormDialog_Si_No_Annulla;
  Frm2 : TFormDialog_Ok;
  Frm3 : TFormDialog_Error;
  Frm4 : TFormDialog_Si_No;
  Ris  : TModalResult;
begin
  SetWindowPos(scheda.Handle,
               HWND_TOPMOST,
               scheda.left,
               scheda.top,
               scheda.width,
               scheda.height,
               SWP_NOACTIVATE);

  Ris := mrCancel;
  if (DialogStyle=TACDS_Si_No_Annulla) then
  begin
    Frm1:=TFormDialog_Si_No_Annulla.Create(Self);
  //  Application.CreateForm(TFormDialog_Si_No_Annulla, Frm1);
    Frm1.D_Titolo.Caption:=DialogTitle;
    Frm1.D_Testo.Caption:=DialogText;
    Ris:=Show_Modal_Dialog(Frm1.ACDialogEngine1);
    Frm1.Free;
  End;
  if (DialogStyle=TACDS_Si_No) then
  begin
    Frm4:=TFormDialog_Si_No.Create(Self);
  //  Application.CreateForm(TFormDialog_Si_No_Annulla, Frm1);
    Frm4.D_Titolo.Caption:=DialogTitle;
    Frm4.D_Testo.Caption:=DialogText;
    Ris:=Show_Modal_Dialog(Frm4.ACDialogEngine1);
    Frm4.Free;
  End;

  if (DialogStyle=TACDS_Ok) then
  begin
    Frm2:=TFormDialog_Ok.Create(Self);
  //  Application.CreateForm(TFormDialog_Si_No_Annulla, Frm1);
    Frm2.D_Titolo.Caption:=DialogTitle;
    Frm2.D_Testo.Caption:=DialogText;
    Ris:=Show_Modal_Dialog(Frm2.ACDialogEngine1);
    Frm2.Free;
  End;
  if (DialogStyle=TACDS_Error) then
  begin
    Frm3:=TFormDialog_Error.Create(Self);
  //  Application.CreateForm(TFormDialog_Si_No_Annulla, Frm1);
    Frm3.D_Titolo.Caption:=DialogTitle;
    Frm3.D_Testo.Caption:=DialogText;
    Ris:=Show_Modal_Dialog(Frm3.ACDialogEngine1);
    Frm3.Free;
  End;

  Show_Dialog_Box := Ris;
end;
end.

