(******************************************************************************
 * AcFileOperation                                                            *
 ******************************************************************************)
unit AcFileOperation;

(******************************************************************************
 * Interface                                                                  *
 ******************************************************************************)
 interface

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
 uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShellAPI;

(******************************************************************************
 * TAcFileOperation                                                           *
 ******************************************************************************)
type
  TACFileOp = (AcCopy,
                     AcDelete,
                     ACMove,
                     AcRename);
  TAcFileOptionFlag = ( AcAllowUndo,
                        AcConfirmMouse,
                        AcFilesOnly,
                        AcMultiDestFiles,
                        AcNoConfirmation,
                        AcNoConfirmMkDir,
                        AcRenameOnCollision,
                        AcSilent,
                        AcSimpleProgress );
  TACSetFileOptionFlag = set of TAcFileOptionFlag;
  TACFileOperation = class(TComponent)
  private
    { Private declarations }
    FData     : TShFileOpStruct; // Struttura usata da ShFileOperation
    fopzioni  : TACSetFileOptionFlag;
    fsorgenti : TStringList; // Elenco file sorgenti
    fdestinazioni : TStringList; // Elenco file destinazione
    foperazione   : TACFileOp; // Operazione da compiere
  protected
    { Protected declarations }
    procedure formatstring(txt1:pChar;List:TStringList);
    function GetDim(List:TStringList):Integer;
    procedure SetFiles(Lista:TStringList);
    procedure SetDestinazioni(Lista:TStringList);
    function GetAzione:Integer;
    function PackFlags:FILEOP_FLAGS;
  public
    { Public declarations }
    Function execute:Integer;
    Constructor Create(AOWner:TComponent);override;
    destructor Destroy; override;
  published
    { Published declarations }
    property SourceFiles : TStringList Read fSorgenti Write SetFiles;
    property DestinationFiles : TStringList Read fDestinazioni Write SetDestinazioni;
    property Action : TACFileOp Read fOperazione Write fOperazione ;
    property Options : TACSetFileOptionFlag read fOpzioni write fOpzioni; 



  end;

(******************************************************************************
 * Implementation                                                             *
 ******************************************************************************)
implementation


(******************************************************************************
 * Execute                                                                    *
 ******************************************************************************)
function TACFileOperation.execute:Integer;
var NomiSourceFile,
    NomiDestFile   : PChar;
Begin
 NomiSourceFile := StrAlloc(GetDim(SourceFiles));
 NomiDestFile   := StrAlloc(GetDim(DestinationFiles));
 formatstring(NomiSourceFile,SourceFiles);
 formatstring(NomiDestFile,DestinationFiles);
 try
   with fData do
   begin
     wnd    := 0;
     wFunc  := GetAzione;
     PFrom  := NomiSourceFile;
     pTo    := NomiDestFile;
     fFlags := PackFlags;
   end;
   result := SHFileOperation(fData);

 finally
   StrDispose(NomiSourceFile);
   StrDispose(NomiDestFile);
 end;
End;

(******************************************************************************
 * FormatString                                                               *
 ******************************************************************************)
procedure TACFileOperation.formatstring(txt1:pChar;List:TStringList);
Var c1, c2, c3 : Integer;
    Txt2 : String;
Begin
 StrPcopy(Txt1,'');
 c3 := 0;
 For c1 := 0 to list.count - 1 do
 Begin
   Txt2 := List.Strings[c1];
   For c2 := 1 to Length(txt2) Do
   Begin
     txt1[c3]:=txt2[c2];
     Inc(c3);
   End;
   txt1[c3]:=#0;
   Inc(c3);
 End;
 txt1[c3]:=#0;
// Inc(c3);
End;

(******************************************************************************
 * GetDim                                                                     *
 ******************************************************************************)
function TACFileOperation.GetDim(List:TStringList):Integer;
var c1, Lun : Integer;
Begin
 Lun := 0;
 For c1 := 0 to List.Count - 1 do
   Lun := Lun + Length(List.Strings[c1])+1;
 Inc(Lun);
 GetDim := Lun;
End;

(******************************************************************************
 * SetFiles                                                                   *
 ******************************************************************************)
procedure TACFileOperation.SetFiles(Lista:TStringList);
Begin
  fsorgenti.Assign(Lista);
End;

(******************************************************************************
 * SetDestinazioni                                                            *
 ******************************************************************************)
procedure TACFileOperation.SetDestinazioni(Lista:TStringList);
Begin
  fdestinazioni.Assign(Lista);
End;

(******************************************************************************
 * GetAzioni                                                                  *
 ******************************************************************************)
function TACFileOperation.GetAzione:Integer;
var op : Integer;
Begin

  if  (Action=AcCopy) Then Op := FO_COPY
  else if  (Action=AcDelete) Then Op := FO_DELETE
  else if  (Action=ACMove) Then Op := FO_MOVE
  Else Op := FO_RENAME;
  GetAzione := Op;
End;

(******************************************************************************
 * PackFlags                                                                  *
 ******************************************************************************)
function TACFileOperation.PackFlags:FILEOP_FLAGS;
var Flag : FILEOP_FLAGS;
Begin
  Flag := 0;
  if (AcAllowUndo          in options) then flag := flag or FOF_ALLOWUNDO;
  if (AcConfirmMouse       in options) then flag := flag or FOF_CONFIRMMOUSE;
  if (AcFilesOnly          in options) then flag := flag or FOF_FILESONLY;
  if (AcMultiDestFiles     in options) then flag := flag or FOF_MULTIDESTFILES;
  if (AcNoConfirmation     in options) then flag := flag or FOF_NOCONFIRMATION;
  if (AcNoConfirmMkDir     in options) then flag := flag or FOF_NOCONFIRMMKDIR;
  if (AcRenameOnCollision  in options) then flag := flag or FOF_RENAMEONCOLLISION;
  if (AcSilent             in options) then flag := flag or FOF_SILENT;
  if (AcSimpleProgress     in options) then flag := flag or FOF_SIMPLEPROGRESS;
  PackFlags:=flag;
End;


(******************************************************************************
 * Create                                                                     *
 ******************************************************************************)
Constructor TACFileOperation.Create(AOWner:TComponent);
Begin
 Inherited Create(AOWner);
 fsorgenti := TStringList.Create;
 fdestinazioni := TStringList.Create;
End;

(******************************************************************************
 * Destroy                                                                    *
 ******************************************************************************)
destructor TACFileOperation.Destroy;
Begin
 fsorgenti.Free;
 fdestinazioni.Free;
 Inherited;
End;


end.
